function fv=calc_friction_vel2(z,u,z1,z0,k,myu)
% Calculate fricition velocity at z corresponding to velocity u at z1
% Solve the equation
% f(y) = x - sqrt(1+y^2)+y*(asinh(y)-A) 
% df/dy = asinh(y)+A 
% y -> y - f(y)/(df/dy)
% y -> (sqrt(1+y^2)-x)/(asinh(y)+A)
%calc_friction_vel2(.6,0,1e-5,1e-6,.41,1.5e-5)
z = z+z0;
z1 = z1+z0;
e = (z1-z)/z;

c = 2*k*z/myu;

B = k*c*u-1+e;
A = log(z/(2*z0));

y = k*c*u/(1+A+log(2)); % Initial guess
y = k*c*u/(1+A-log(y)); % Initial guess
 
con_tol=1e-10;
maxit = 10;
for n=1:maxit
  oy = y;
  sy = sqrt(y^2+1);
  y = (B+sy-y-e/sy)/(log(1+sy/y)+A+e*y/sy);
  if(abs(y-oy)<=con_tol) 
    break
  end
end

sy = sqrt(1+y^2);
nu = (1-sy + y*(log(1+sy/y) +1+A))/(c*k);
du = (sy-1)/(k*c*z);
fv = sqrt(abs(du)*(myu+k^2*z^2*abs(du)));
y2  = 2*k*fv/myu*z;
sy2 = sqrt(1+y2^2);
nnu = y2*(2/(1+y2+sy)+log((y2+sy2)/y2)-log(2*(z0/z)))/(c*k)
disp(['Iterations=',num2str(n)]);
disp(['u=',num2str(nu),',du=',num2str(du)]);
disp(['nu+(z1-z)*du-u=',num2str(nu+(z1-z)*du-u)]);
disp(['u-f(du)=', num2str(nnu-nu)]);
  
% $$$ keyboard
% $$$ fv = sqrt(abs(du)*(myu+k^2*z^2*abs(du)));
% $$$ if abs(fv-y/c)>eps
% $$$   error('Friction Velocity and gradient are wrong');
% $$$ end
% $$$ 
% $$$ keyboard
