function [nv,nx,dt,my]=calc_lindrag_trajs(v,x)
% Calculate trajectories for linear air drag with constant air
% velocity

if nargin<2
  x = zeros(size(v));
end

global gravity
global air_viscosity air_frictionv
global grain_density grain_diam

air_velocity = [air_frictionv;0];

drag_c = 18*air_viscosity/(grain_density*grain_diam^2);

acc = gravity+drag_c*air_velocity;

%equation is ddx/dtt = acc - drag_c dx/dt) 

dt = impact_t(x(2,:),v(2,:),acc(2,:),drag_c);
tt = apogee(v(2,:),acc(2,:),drag_c);

my = pos(x(2,:),v(2,:),acc(2,:),drag_c,tt);

[nx nv] = pos(x,v,acc,drag_c,dt);

% Trajectory functions
% Make sure works for l = 0 corresponding to no air drag.

function [p,v] = pos(x0,v0,acc,l,t)

if (l==0)
  t1 = t;
  t2 = t.^2/2;
else
  t1 = (1-exp(-l*t))/l;
  t2 = (exp(-l*t) + t*l - 1)/l.^2; 
end
d = size(x0,1); % How many dimensions

t11 = repmat(t1,[d 1]);

p = x0 + v0.*t11 + acc*t2;
v = v0 + acc*t1 - l*v0.*t11;

% Time of maximum y
function t = apogee(v0,acc,l)
if (l==0);
  t = -v0./acc;
else
  t = log(1-v0.*l./acc)./l;
end



function t = impact_t(x0,v0,acc,l)

% solution is between
% (-(sqrt(-2*x0*f+v0^2)+v0)/f)+O(l)
% -v0/f+1/l+O(exp(l*v0/f))
% Estimate time ignoring drag
t = 2*apogee(v0,acc,l);

n = 0;
while n<5
  n=n+1;
  [p v]=pos(x0,v0,acc,l,t);
  t = t - p./v;
end

%vi=20*rand(1);c=rand(1);g=-10*rand(1);[p v]=pos(0,vi,g,c,apogee(vi,g,c));disp(v)

%y=rand(1);vi=20*rand(1);c=rand(1);g=-10*rand(1);pos(y,vi,g,c,impact_t(y,vi,g,c))

