function [vin, vout, dp]=display_traj(name)

if nargin <1
  trj = read_traj;
else
  trj = read_traj(name);
end

% First sort out particle numbering
p = find(trj.parent~=0);        % These are the grains with trj.parents
n = length(trj.parent);
ap = NaN*zeros(1,max(trj.num));
ap(trj.num) = 1:length(trj.num);
np(p) = ap(trj.parent(p));
if any(trj.num(np(p)) ~= trj.parent(p))
  error('Messed up parents')
end

aero     = find(trj.origin==1);
ricochet = find(trj.origin==2);
splash   = find(trj.origin==3);
if length(p)~=length(ricochet)+length(splash)
  disp('Trj.Parents and particle types are wrong.')
end
trj.num_splash  = zeros(1,n);
trj.splash(:,:) = zeros(10,n);
trj.ricochet    = zeros(1,n);
trj.ricochet(np(ricochet)) = 1; % Which ones ricoched
for i=1:length(splash)
  j = np(splash(i));         
  trj.num_splash(j) = trj.num_splash(j)+1;
  trj.splash(trj.num_splash(j),j)=splash(i);
end
pp = zeros(1+max(trj.num_splash),1);
for i=1:n
  pp(1+trj.num_splash(i))=pp(1+trj.num_splash(i))+1;
end

set(0,'defaultlinemarkersize',1) 

figure(1)

% $$$ subplot(2,2,1)
% $$$ cla
% $$$ if length(pp)>2
% $$$   bar(1:length(pp(2:end)),pp(2:end)/n);
% $$$ end
% $$$ title('Number of splashed particles');
% $$$ ylabel('Relative frequency')
% $$$ xlabel('Incoming velocity');

subplot(2,2,1)
plot(trj.start_vel(1,splash),trj.start_vel(2,splash),'.')
title('Initial velocities splash')
ylabel('y velocity');
xlabel('x velocity');

subplot(2,2,2)
plot(trj.start_vel(1,ricochet),trj.start_vel(2,ricochet),'.')
title('Initial velocities ricochet')
ylabel('y velocity');
xlabel('x velocity');

subplot(2,2,3)
vsqr = sqrt(sum(trj.end_vel.^2));
[m_num s_num v_bin]=dist_hist(vsqr,trj.num_splash);
bar(v_bin,m_num,'hist');
ylabel('Mean number');
xlabel('Incoming velocity');
title('Splashed grains');

subplot(2,2,4)
[m_num s_num v_bin]=dist_hist(vsqr,trj.ricochet);
bar(v_bin,m_num,'hist');
ylabel('Probability');
xlabel('Incoming velocity');
title('Ricochet probability');

figure(2)
plot_collision(trj.end_vel(:,np(ricochet)),trj.start_vel(:,ricochet),'Ricochet')
figure(3)
plot_collision(trj.end_vel(:,np(splash)),trj.start_vel(:,splash),'Splash')
