function display_saltout(name,phase,filt)
%phase=1 make phase space plots, otherwise plot against time
if nargin < 3
  filt=1; % Size of filter
end
if nargin < 2
  phase = 0;
end
if nargin < 1
  name = '';
end

if isempty(name)
  name = '/home/jim/saltare/out/saltout.dat';
end
reclen=13;
fp=fopen(name,'r');
a = fscanf(fp,'%f');
fclose(fp);
n = floor(length(a)/reclen);
if reclen*n~=length(a)
  disp('warning: file is incomplete')
end
a = reshape(a(1:reclen*n),reclen,n);
a = moving_avg(a,filt,2); % Filter all quantities
iter   = a(1,:);
time   = a(2,:);
bot_fv = a(3,:);
top_fv = a(4,:);
bot_v  = a(5,:);
top_v  = a(6,:);
z0     = a(7,:);
num    = a(8,:);
mass   = a(9,:);
mom    = a(10,:);
imp    = a(11,:); 
down   = a(12,:); 
up     = a(13,:); 

if phase==0
  np = 7;
  subplot(np,1,1);
  plot(time,bot_fv,time,top_fv)
  title('Top and bottom wind shear velocity against time')
  ylabel('m/s')
  axis tight
  a=axis;
  axis([a(1:2) 0 a(4)]);

  subplot(np,1,2);
  plot(time,bot_v,time,top_v)
  title('Top and bottom wind velocity against time')
  ylabel('m/s')
  axis tight
  a=axis;
  axis([a(1:2) 0 a(4)]);

  subplot(np,1,3);
  plot(time,z0)
  title('Rougness height against time')
  ylabel('m')
  axis tight

  subplot(np,1,4);
  plot(time,imp)
  title('Grain stress against time')
  ylabel('N/m^2')
  axis tight

  subplot(np,1,5);
  plot(time,down,time,up,time,up-down)
  title('Vertical fluxes, up, down and net against time')
  ylabel('Kg/m^2/s')
  axis tight

  subplot(np,1,6);
  plot(time,mass)
  title('Mass load against time')
  ylabel('Kg/m^2')
  axis tight

  subplot(np,1,7);
  plot(time,mom)
  title('Mass flux against time')
  ylabel('Kg/m/s')
  axis tight
else
  
  [a n2]=max(top_fv);                % Maximum point
  n1=max(find(mom(1:n2)==0));        % Continuous threshold
  n4=min(find(mom(1:n2)>0));         % Continuous threshold
  n3=n2-1+min(find(mom(n2:end)==0)); % Aero threshold
  n5=n2-1+max(find(mom(n2:end)>0));  % Final threshold
  afv   = top_fv(n1:n2);
  bfv   = top_fv(n2:n3);
  az0   = z0(n1:n2);
  bz0   = z0(n2:n3);
  aimp  = imp(n1:n2);
  bimp  = imp(n2:n3);
  amass = mass(n1:n2);
  bmass = mass(n2:n3);
  amom  = mom(n1:n2);
  bmom  = mom(n2:n3);

  clf
  subplot(2,2,1);
  plot(afv,az0,bfv,bz0)
  
  title('Rougness height against friction velocity')
  ylabel('m')
  xlabel('m/s')
  axis tight

  subplot(2,2,2);
  plot(afv,aimp,bfv,bimp)
  title('Grain stress against friction velocity')
  ylabel('N/m^2')
  xlabel('m/s')
  axis tight

  subplot(2,2,3);
  plot(afv,amass,bfv,bmass)
  title('Mass load against friction velocity')
  ylabel('Kg/m^2')
  xlabel('m/s')
  axis tight

  subplot(2,2,4);
  cafv = polyfit(afv,amom,2);
  cbfv = polyfit(bfv,bmom,2);
  plot(afv,amom,bfv,bmom,afv,polyval(cafv,afv),bfv,polyval(cbfv,bfv))
  
  title('Mass flux against friction velocity')
  ylabel('Kg/m/s')
  xlabel('m/s')
  axis tight
end



