function [vin, vout, dp]=display_traj(name)

if nargin <1
  trj = read_traj;
else
  trj = read_traj(name);
end

aero = find(trj.origin==1);
ricochet = find(trj.origin==2);
splash = find(trj.origin==3);
p = find(trj.parent~=0);        % These are the grains with trj.parents

if length(p)~=length(ricochet)+length(splash)
  disp('Trj.Parents and particle types are wrong.')
end
% Find the trj.parents
ap = NaN*zeros(1,max(trj.num));
ap(trj.num) = 1:length(trj.num);
np(p) = ap(trj.parent(p));
if any(trj.num(np(p)) ~= trj.parent(p))
  error('Messed up parents')
end

set(0,'defaultlinemarkersize',1) 

figure(1)

subplot(2,2,1)
plot(trj.start_vel(1,aero),trj.start_vel(2,aero),'.')
title('Initial velocities aero')
subplot(2,2,2)
plot(trj.start_vel(1,ricochet),trj.start_vel(2,ricochet),'.')
title('Initial velocities ricochet')
subplot(2,2,3)
plot(trj.start_vel(1,splash),trj.start_vel(2,splash),'.')
title('Initial velocities splash')
subplot(2,2,4)
plot(trj.start_vel(1,aero),trj.start_vel(2,aero),'.',...
    trj.start_vel(1,ricochet),trj.start_vel(2,ricochet),'.',...
    trj.start_vel(1,splash),trj.start_vel(2,splash),'.')
%plot(trj.start_vel(1,:),trj.start_vel(2,:),'.')
title('Initial velocities all')
trj.start_an = atan2(trj.start_vel(2,:),trj.start_vel(1,:))*180/pi;
trj.end_an = atan2(trj.start_vel(2,:),trj.start_vel(1,:))*180/pi;


figure(2)
vsqr = sqrt(trj.start_vel(1,ricochet).^2+trj.start_vel(2,ricochet).^2);
vang = atan2(trj.start_vel(2,ricochet),trj.start_vel(1,ricochet))*180/pi;
in = np(ricochet);
vsqrp = sqrt(trj.end_vel(1,in).^2+trj.end_vel(2,in).^2);
vangp = -atan2(trj.end_vel(2,in),trj.end_vel(1,in))*180/pi;

subplot(2,2,1)
plot(vangp,vsqr./vsqrp,'.')
xlabel('In angle')
ylabel('Total restitution')
a = axis; axis([0 180 a(3) a(4)]);
title('Ricochet restitution as a function of angle')
subplot(2,2,3)
plot(vangp,trj.start_vel(1,ricochet)./trj.end_vel(1,in),'.')
xlabel('In angle')
ylabel('Horizontal restitution')
title('Ricochet horizontal restitution as a function of angle')
a = axis; axis([0 180 max([-5 a(3)]) min([5 a(4)])]);
subplot(2,2,4)
plot(vangp,-trj.start_vel(2,ricochet)./trj.end_vel(2,in),'.')
xlabel('In angle')
ylabel('Vertical restitution')
title('Ricochet vertical restitution as a function of angle')
a = axis; axis([0 180 max([-5 a(3)]) min([5 a(4)])]);
subplot(2,2,2)
plot(vangp,vang,'.')
xlabel('In angle')
ylabel('Out angle')
title('Ricochet out angle as a function of in angle')
a = axis; axis([0 180 0 180]);

figure(3)
subplot(3,1,1)
plot_hist(trj.max_y(aero))
title('Maximum height histogram of aero particles')
subplot(3,1,2)
if length(ricochet>0)
  plot_hist(trj.max_y(ricochet))
  title('Ricochet')
  title('Maximum height histogram of ricochet particles')
end
subplot(3,1,3)
if length(splash>0)
  plot_hist(trj.max_y(splash))
  title('Splash')
  title('Maximum height histogram of splash particles')
end
