      SUBROUTINE CALCU(t,shear,sp,gstr,test)
!----------------------------------------------------------------------
!            U-Velocity calculation
!     implicit scheme. Mixing length. unsteady.
!
!     input:  time, pressure gradient, particle
!             sources for momentum X-change
!     output: shear stress, new U-field
!
!     mixing length model
!----------------------------------------------------------------------
      INCLUDE 'params.inc'
      INCLUDE 'pars.inc'
      INCLUDE 'discrete.inc'
!...               ^------ defines    NJ=NYMAX
      INCLUDE 'ufluct.inc'
!                this is for:       ubase,ampli,period,eps,maxit
      INCLUDE 'timing.inc'
      INCLUDE 'strecom.inc'
      INCLUDE 'tracom.inc'
      INCLUDE 'ucalc.inc'

      REAL k2,vis,viss,visn,norm,t,un,us,du,yint
      REAL ap,as,an,sp,s,a,q,uo,shear,gstr
      REAL du1,du2,dun,dus, dus1
      LOGICAL test

      INTEGER j,inner
      DIMENSION ap(nymax),as(nymax),an(nymax),sp(nymax),uo(nymax)&
           ,s(nymax),a(nymax),q(nymax),shear(nymax)&
           ,gstr(nymax)
!      DATA s,uo,as,an,ap/nj*0.,nj*0.,nj*0.,nj*0.,nj*0./
      COMMON /added_mass/ uo                  ! later a `vector'
 

      k2=vkarman**2
      vis=visco/fdens

!...      outer flow velocity:
!         Different types!!!
      SELECT CASE (type)
        CASE (1)
          fluid.U(nj)=ubase+ampli*sin(2.*pi*t/period)
        CASE (2)
          fluid.U(nj)=ubase+sign(ampli,&
          real(mod(1+nint(real(2.,r_kind)*t/period),2),r_kind)-1.)
        CASE (4)
          fluid.U(nj)=ubase+(uend-ubase)*float(iter)/numiter
        CASE (5)
          fluid.U(nj)=ubase
          IF (iter.gt.itstatio)&
           fluid.U(nj)=ubase+(uend-ubase)*float(iter-itstatio)/(numiter-itstatio)
      END SELECT
!
!.... press is dP/dx (negative!), dmom.x=particle source term
      DO j=2,njm
        sp(j)=urf*(-dmom(j).x/(tstep*area*fdens*(grid.y(j)-grid.y(j-1))) -press/fdens)&
             +(1.-urf)*sp(j)
      ENDDO
!
!     TEST OUTPUT
      IF (test) write(*,'(/,i5,4(/7(1x,e10.3)))') iter,(sp(j),j=1,min(nymax,28))
      IF (test) write(*,'(4(/7(1x,i10))/)') (number(j),j=1,min(nymax,28))

      DO j=1,nj
        uo(j)=fluid.U(j)   ! uo() is also needed for added mass/pressure term in TRACK
        s(j)=fluid.U(j)+max(real(0.,r_kind),sp(j))*tstep  !-s() is source vector for matrix solver
      ENDDO

      inner=0
      norm=0.

      DO
      inner=inner+1
!---- calculate coefficients
!     magnitude of updated fluid.U()
      norm=0.

!---  south boundary
      dun=2.*abs(fluid.U(3)-fluid.U(2))/(grid.y(3)-grid.y(1))
      dus1=abs(fluid.U(2)-fluid.U(1))/(0.5*grid.y(2)-grid.y(1))
      dus=mixf*dus1+(1.-mixf)*dun
      IF (dun.lt.dudymin) THEN
        dun=2.*abs(uo(3)-uo(2))/(grid.y(3)-grid.y(1))
      ENDIF
      IF (dus.lt.dudymin) THEN
        dus1=abs(uo(2)-uo(1))/(0.5*grid.y(2)-grid.y(1))
        dus=mixf*dus1+(1.-mixf)*dun
      ENDIF
      visn=vis+k2*(grid.y(2)+yzero)**2*dun
      viss=vis+k2*(grid.y(1)+yzero)**2*dus
      an(2)=visn*tstep/((grid.y(2)-grid.y(1))*(0.5*grid.y(3)-grid.y(1)))
      as(2)=viss*tstep/((grid.y(2)-grid.y(1))*(0.5*grid.y(2)-grid.y(1)))
      ap(2)=1.+an(2)+as(2)+max(real(0.,r_kind),-sp(2)*tstep/(fluid.U(2)+1.e-20))
! test      ap(2)=1.+an(2)+as(2)+max(0.,-sp(2)*tstep/(uo(2)+1.e-20))

      DO j=3,njm
        viss=visn
        dun=2.*abs(fluid.U(j+1)-fluid.U(j))/(grid.y(j+1)-grid.y(j-1))
        IF (dun.lt.dudymin) dun=2.*abs(uo(j+1)-uo(j))/(grid.y(j+1)-grid.y(j-1))
        visn=vis+k2*(grid.y(j)+yzero)**2*dun
        an(j)=2.*visn*tstep/((grid.y(j+1)-grid.y(j-1))*(grid.y(j)-grid.y(j-1)))
        as(j)=2.*viss*tstep/((grid.y(j)-grid.y(j-2))*(grid.y(j)-grid.y(j-1)))
        ap(j)=1.+an(j)+as(j)+max(real(0.,r_kind),-sp(j)*tstep/(fluid.U(j)+1.e-20))
! test        ap(j)=1.+an(j)+as(j)+max(0.,-sp(j)*tstep/(uo(j)+1.e-20))
      ENDDO

      CALL TDMA(nj,fluid.U,ap,an,as,a,q,s)

      norm=0.              ! in F90 :   norm=SUM((fluid.U(j)-uo(j))**2)
      DO j=1,nj
        norm=norm+(fluid.U(j)-uo(j))**2
      ENDDO

      IF (inner .eq. maxit .or. norm .lt. eps) EXIT  ! the big loop
      ENDDO

      DO j=3,njm
        yint=.5*(grid.y(j)+grid.y(j-1))
        un=fluid.U(j+1)*grid.fy(j)+fluid.U(j)*(1.-grid.fy(j))
        us=fluid.U(j)*grid.fy(j-1)+fluid.U(j-1)*(1.-grid.fy(j-1))
        du=(un-us)/(grid.y(j)-grid.y(j-1))
        dun=abs(du)
        IF (dun.lt.dudymin) THEN
          un=uo(j+1)*grid.fy(j)+uo(j)*(1.-grid.fy(j))
          us=uo(j)*grid.fy(j-1)+uo(j-1)*(1.-grid.fy(j-1))
          du=(un-us)/(grid.y(j)-grid.y(j-1))
          dun=abs(du)
        ENDIF
        shear(j)=fdens*k2*yint**2*dun*du
      ENDDO
!
      yint=0.5*grid.y(2)+yzero
      du2=2.*(fluid.U(3)-fluid.U(2))/grid.y(3)
      du1=2.*(fluid.U(2)-fluid.U(1))/(grid.y(2)-grid.y(1))
      du=mixf*du1+(1.-mixf)*du2
      dun=abs(du)
      IF (dun.lt.dudymin) THEN
        un=uo(3)*grid.fy(2)+uo(2)*(1.-grid.fy(2))
        us=uo(2)
        du=(un-us)/(grid.y(2)-grid.y(1))
        dun=abs(du)
      ENDIF
      shear(2)=fdens*k2*yint**2*dun*du
      shear(1)=visco*2.*fluid.U(2)/grid.y(2)
      shear(nj)=shear(njm)
!
!-----grain borne stress for profile output in saltare.f
      gstr(1)=dmom(1).x/(tstep*area)
      DO j=2,nj
        gstr(j)=gstr(j-1)+dmom(j).x/(tstep*area)
      ENDDO

      END
