module mid_air_mod
  use precision
  use discrete
  use particle
  use sysdep_io
  use maths_const
  private
  public :: check_mid_air

  real(r_kind), private :: totnmid
!  real(r_kind), private :: wfrac,vfrac
!  integer, private ::  nmid(1000,0:50)
!  real(r_kind), private ::  wt1,wt2,wt3,vt1,vt2,vt3

  logical, parameter, private :: mid_air_collisions = .false.
  logical, parameter, private :: test_mid_air = .true.
  integer, private :: collis_count=0

contains
  subroutine check_mid_air(g,ig,rnumb,dt)
    type(particles), intent(inout) :: g(:) !Length is total number of grains
    integer, intent(in) :: ig
    real(r_kind), intent(in) :: rnumb(:) ! Particle volume density
    real(r_kind), intent(in) :: dt ! Time step

    integer gp
    logical found
    real(r_kind) d, tmp_rand
    real(r_kind) pcoll ! Probability of collision
    integer igdel

    if(.not. mid_air_collisions) return

    found = .false.

    gp = g(ig)%gy  ! Grid coordinate
    d = g(ig)%diam ! grain diameter 

    ! Calculate collision probability
    pcoll=rnumb(gp)/grid_vol(gp)*pi*d*d*dt&
         *sqrt(g(ig)%vel%x**2+g(ig)%vel%y**2)
    call random_number(tmp_rand)
    if (pcoll>tmp_rand) then
!       nmid(iter,0)=nmid(iter,0)+1
!       nmid(iter,gp)=nmid(iter,gp)+1
       collis_count=collis_count+1
       if (test_mid_air) write(std_out,'(a,i3)') 'collis_count',collis_count
       do igdel = 1,size(g)
          if (igdel==ig) cycle
          if(g(igdel)%gy == gp) then
             found = .true.
             exit
          end if
       end do
       call assert(found,&
            "mid_air_mod.f90 Couldn't find a particle at the same height")
       call midair(g(ig),g(igdel))
    end if
  end subroutine check_mid_air

  ! Mid air collision between two grains
  ! Looks rather strange to me but...

  subroutine midair(g1,g2)
     type(particles), intent(inout) :: g1
    type(particles), intent(inout) :: g2

    dimension h(3),w(3),v(3)

    !   adopt the notation and concepts given in sorensen
    !   and mcewan (1994)
    !   note that l is declared as real
    integer i
    real(r_kind) up,vp,r,l,u,h,w,a,b,psi,v,ru,xl

    up = g1%vel%x-g1%vel%x
    vp = g1%vel%y-g1%vel%y
    u=sqrt(up*up+vp*vp)

    psi=atan(-vp/up)
    r=g1%diam/2
    call random_number(ru)
    i=-1
    do
       i=i+1
       l=2*r*real(i,r_kind)/100.
       xl=l*l/(4*r**2)
       if(ru<=xl) exit
    end do

    b=asin(l/(2*r))
    call random_number(a)
    a=a*2*pi

    h(1)=0.85*(cos(psi)*cos(b)*cos(b)+sin(psi)*cos(b)*sin(b)*sin(a))
    h(2)=0.85*(-sin(psi)*cos(b)*cos(b)+cos(psi)*cos(b)*sin(b)*sin(a))
    h(3)=0.85*cos(b)*sin(b)*cos(a) 
    w(1)=u*h(1)
    w(2)=u*h(2)
    w(3)=u*h(3) 
    v(1)=u*(cos(psi)-h(1))
    v(2)=u*(-sin(psi)-h(2))     
    v(3)=-u*h(3)
    totnmid=totnmid+1.
    g1%vel%x=v(1)+g2%vel%x
    g1%vel%y=v(2)+g2%vel%y
    g1%acc%x=0
    g1%acc%y=0

    g2%pos%x=g1%pos%x
    g2%pos%y=g1%pos%y
    g2%vel%x=w(1)+g2%vel%x
    g2%vel%y=w(2)+g2%vel%y
    g2%acc%x=0
    g2%acc%y=0
    g2%diam=g1%diam

    g2%gy=g1%gy
  end subroutine midair
end module mid_air_mod
