module reading
  use params
   use particle
  use ucalc
  use discrete
  use splsh
  use ricochmod
  use gammas
  use ufluct
  use timing 
  use pszdstr
  use outdirmod
  use sysdep_io

  real(r_kind) sstt
  integer ndum
  integer nmeas
  integer itest(5)
  logical onesize
  character(len=20) velocin
contains
  subroutine read_all(casefil,read3,write3,history,profs& 
       ,test,scrout,filin,filout)
    !****************************************************************
    !     read in input parameters
    !****************************************************************
    character(len=*), intent(in) :: casefil
    character(len=*), intent(out) :: filin
    character(len=*), intent(out) :: filout
    logical, intent(out) :: read3,write3,history,profs,test,scrout
 
    integer temp_fp
    integer input_fp
    integer j

    call assign_lun(input_fp) 

    write(std_out,'(1x,2a)') ' opening input file ',casefil

    call assign_lun(input_fp)
    open(input_fp,file=casefil,status='old')
    call task_param(input_fp)
    call switch_param(input_fp)
    call output_param(input_fp)
    call environment_param(input_fp)
    call bed_param(input_fp)
    !+++++ the grains:
    call bed_impact_param(input_fp)
    call saltation_param(input_fp)
    call grain_param(input_fp)
    call entr_vel_param(input_fp)
    call sand_param(input_fp)
    close(input_fp);call free_lun(input_fp)
    write(std_out,'(2x,2a)') casefil,' closed.'

    !.... initial wind profile:
    if ( .not. read3) then
       call assign_lun(temp_fp)
       write(std_out,'(14x,2a)')  '... now opening ',velocin
       open(temp_fp,file=velocin,form='unformatted',status='old')
       call read_wind_profile(temp_fp)
       close(temp_fp);call free_lun(temp_fp)
       write(std_out,'(2x,2a)') velocin,' closed.'
    endif

    write(std_out,'(1x,a)') ' opening grid.bin...'
    call assign_lun(temp_fp)
    open(temp_fp,file='grid.bin',form='unformatted',status='unknown')
    call read_grid_bin(temp_fp)
    close(temp_fp);call free_lun(temp_fp)
    write(std_out,*) ' grid file read. opening velo_in'

    !
    !---- read in parameters from file(s)
    !
    call assign_lun(temp_fp)
    open(temp_fp,file='velo_in',status='old')
    call read_velo_in (temp_fp)
    close(temp_fp);call free_lun(temp_fp)
    write(std_out,*) ' velo_in closed. reading gamma.bin'

    call assign_lun(temp_fp)
    open(temp_fp,file='gamma.bin',status='old',form='unformatted')
    call read_gamma_bin(temp_fp)
    close(temp_fp);call free_lun(temp_fp)
    write(std_out,*)' gamma.bin closed. reading sur.bin'

    area=bedlen*bedwid
    vol(1)=0.
    do j=2,njm
       vol(j)=area*(grid%y(j)-grid%y(j-1))
       airmss(j)=vol(j)*fdens
    enddo
    vol(nj)=0.

  
    call assign_lun(temp_fp)
    open(temp_fp,file='sur.bin',form='unformatted')
    call read_sur_bin(temp_fp)
    close(temp_fp);call free_lun(temp_fp)

    deltat=sstt*tstep
    !
    !---- for batch listing:
!!$    write(*,'(//1x,a)')    ' this run with...'
!!$    write(std_out,101) &
!!$         ' iterations:',numiter &
!!$         ,' samples from ',samsta,' to ',samend &
!!$         ,' succsalt  :',succsalt,'      plife:',plife &
!!$         ,' bedlength :',bedlen,'  bedwidth :',bedwid &
!!$         ,' entfac    :',entfac,'   profile :',xprof &
!!$         ,' base time :',tstep, 's  base/dt :',1./sstt &
!!$         ,' dt_0      :',deltat,'s  bed slop:',slope 
!!$    !     &     ,' epsilon   :',ep1,ep2,ep3
!!$    if (history) write(std_out,102)&
!!$         ' time logging every ',histout,' iterations (f)'
!!$    if (profs) write(std_out,102)&
!!$         ' writing prof every ',profout,' iterations (p)'
!!$    write(std_out,'(1x,2(a,l1))') &
!!$         ' read in: ',read3,' write out: ',write3
!!$    write(std_out,111)
    !      write(*,*)
    !      write(*,'(1x,a)') '        hit return'
    !      read(*,*)

101 format(1x,a12,i12,/1x,2(a,i5),& 
         /5(1x,2(a12,e12.5),:,/)& 
         !     &      /1x,a12,3(f4.1,:,', ')    ! that's for the ep1, ep2, ...
    )
102 format(1x,a,i2,a)
111 format(2x,50('_'))

  end subroutine read_all
  subroutine  switch_param (fp)
    integer, intent(in) :: fp
    !---- switches
    read(input_fp,*) onesize, history, profs, test, scrout
    read(input_fp,*) (itest(i),i=1,5)
    read(input_fp,*) read3, filin
    read(input_fp,*) write3, filout
    read(input_fp,*) type,ubase,uend, itstatio
    read(input_fp,*) ampli,period,maxit,eps,dudymin
    read(input_fp,*) press, mixf, urf
  end subroutine switch_param
  subroutine  output_param (fp)
    integer, intent(in) :: fp
    !---- output specs
    read(input_fp,*) outdir,velocin
    outdir = trim(outdir) // '/'
    write(*,*) ' output directory is:',trim(outdir),' !'
  end subroutine output_param
  subroutine  environment_param (fp)
    integer, intent(in) :: fp  
    real(r_kind) gravity
    !---- the environment (fluid and constant of gravity )
    read(input_fp,*) fdens, visco, gravity
    gravit%x=0.
    gravit%y=gravity
  end subroutine environment_param
  subroutine  bed_param (fp)
    integer, intent(in) :: fp
    !---- the bed
    read(input_fp,*) bedlen, bedwid, xprof, slope
  end subroutine bed_param
  subroutine  bed_impact_param (fp)
    integer, intent(in) :: fp
    !----- bed impact specs
    read(input_fp,*) succsalt,plife
  end subroutine bed_impact_param
  subroutine  saltation_param (fp)
    integer, intent(in) :: fp
    !---- saltation experiments (define the sand bed)
    read(input_fp,*) tauthres,entfac,poissa,yzero
  end subroutine saltation_param
  subroutine  grain_param (fp)
    integer, intent(in) :: fp
    !---- grains density and sizes
    read(input_fp,*) grain_dens
    read(input_fp,*) (grnsz(i),i=1,nsz)
    do i=1,nsz
       grnsz(i)=1.e-6*grnsz(i)
    enddo
  end subroutine grain_param
  subroutine  entr_vel_param (fp)
    !---- entrained grains' velocities
    read(input_fp,*) uav,u50,u0,vav,v50,v0
    integer, intent(in) :: fp
  end subroutine entr_vel_param
  subroutine  sand_param (fp)
    integer, intent(in) :: fp
    !---- sand characteristics:
    read(input_fp,*)  nmeas
    mesh(0)=0.
    do i=1,nmeas
       read(input_fp,*) mesh(i),cumul(i)
    enddo
  end subroutine sand_param
  subroutine  read_wind_profile (fp)
    integer, intent(in) :: fp
    read(temp_fp) fluid%u,tau
  end subroutine read_wind_profile
  subroutine  read_grid_bin(fp)
    integer, intent(in) :: fp
    read(temp_fp) ndum
    read(temp_fp) (grid%y(j),j=1,ndum)
    read(temp_fp) (grid%fy(j),j=1,ndum)
    if (ndum/=nymax) then
       write(*,'(1x,a,/,1x,a)')& 
            'grid size <> nymax! problems will occur in subroutine calcu.',& 
            'set nymax to the appropriate value in params%inc and recompile.'
       stop
    endif
  end subroutine read_grid_bin
  subroutine  read_velo_in (fp)
    read(icvel,*) (ui(i),i=1,nu)
    read(icvel,*) (vj(i),i=1,nv)
    integer, intent(in) :: fp
  end subroutine read_velo_in
  subroutine  read_gamma_bin (fp)
    integer, intent(in) :: fp
    read(icgam) (gamma(i),i=1,nsplsh)
  end subroutine read_gamma_bin
  subroutine read_gamam
    subroutine  read_sur_bin (fp)
      !---- depending on the order of input data in 'sur.bin' change these
      !            if necessary:
      integer, intent(in) :: fp
      hmax=-100.0
      do i=0,nix-1
         read(icsur,end=20) sur(i+1)%x, sur(i+1)%y, sur(i+1)%d
         if (sur(i+1)%y > hmax) hmax=sur(i+1)%y
      enddo

20    i=min(i,nix)
      write(std_out,'(1x,a,i6,a)') ' end of file in: sur.bin! ('&  
           ,i,' records read)'
      hmax=hmax+1.0
    end subroutine read_sur_bin

  end module reading
