      SUBROUTINE RICOCH(x,r,v1,a1,v3,a3)
!**********************************************************************
!     Ricocheting grains in multiple grain size bed.
!     input:
!           X,R   : particle position and Radius (!)
!           V1,A1 : Velocity and Angle
!     output:
!           V3,A3 : Velocity and Angle
!
! --->  switch length units into mm in this routine for compatibility
!       with bed model
!**********************************************************************
      INCLUDE 'params.inc'
      INCLUDE 'pars.inc'
      INCLUDE 'ricochmod.inc'
      INCLUDE 'ichan.inc'

      REAL x,r,xp,rp,v1,a1,v3,a3,xs,rlen,xn1,dh,zn1,xn,zn,znn
      REAL x0,y0,rm,r0,co,c1,c2,c3,b24ac,x11,y11,x12,y12
      REAL x2,y2,beta
      INTEGER ix,numx

      xp=1000*x
      a1=-a1
    
      rp=1000*r
      rlen=1000*bedlen

      xs=xp-hmax/tan(a1)
!      xs=xp-max(hmax/sign(1.e-30+abs(tan(a1)),tan(a1)),bedlen)
     
      IF (xs .lt. 0.) xs=xs+rlen
      if (xs .lt. sur(1).x) THEN
           ix=nix
      ELSE IF (xs .ge. sur(nix).x) THEN
           ix=1
      ELSE
        ix=0     !   find impact point's position
        DO
          IF (sur(ix+1).x .gt. xs) EXIT
          ix=ix+1
        ENDDO

      ENDIF
!
!...  this takes xn1 to be x pos of lowest point of incoming grain
      xn1=sur(ix).x-rp*sin(a1)
      IF (xn1 .lt. 0.0) xn1=xn1+rlen
      dh=(xs-sur(ix).x)*tan(a1)
      IF (ix .eq. nix) dh=(xs+rlen-sur(ix).x)*tan(a1)
      zn1=hmax+dh-rp*cos(a1)

   15 CONTINUE
        ix=ix+1
        IF (ix .gt. nix) THEN
          ix=1
          xn1=xn1-rlen
        ENDIF
        xn=sur(ix).x+sur(ix).d*sin(a1)
        IF (xn .gt. rlen) xn=xn-rlen
        zn=sur(ix).y+sur(ix).d*cos(a1)
!....   this takes initial projection point to lowest part of a grain
        znn=zn1-(xn-xn1)*tan(a1)
        IF (znn .lt. zn) GOTO 16
      GOTO 15
   16 CONTINUE
      numx=ix
!
!---- exact coll point
!
      x0=sur(numx).x-(xn1+rp*sin(a1))
      IF (x0 .lt. 0.0) x0=x0+rlen
      y0=sur(numx).y
      r0=sur(numx).d
      rm=-tan(a1)
      co=hmax+dh
      c1=1.+rm**2
      c2=2.*(rm*(co-y0)-x0)
      c3=x0**2 +(co-y0)**2 -(rp+r0)**2
      b24ac=c2**2-4.*c1*c3
      b24ac=max(0.,b24ac)
      x11=-0.5*(c2+sqrt(b24ac))/c1
      y11=rm*x11+co
      x2=x11
      y2=y11
      beta=-1.*atan((x0-x2)/(y0-y2))
      IF (y0 .gt. y2) THEN
         beta=beta+pi
      ENDIF
!---  i.e. impacting grains strikes underside of target grain
      a3=2.*beta+a1
!      e=0.6
      v3=0.6*v1
      IF (a3 .lt. -a1 .or. a3 .ge. pi) a3=a1 
!          THEN
!        write(icout,100)
!     &    'a1:',a1*57.3,'  a3:',a3*57.3,' beta:',beta*57.3
!     &    ,'c1,c2,c3,co,rm',c1,c2,c3,co,rm
!     &    ,'c2c2-4c1c3',c2*c2-4*c1*c3
!     &    ,'x0:',x0,'y0:',y0,'r0:',r0,'x2:',x2,'y2:',y2,'rp:',rp
!     &    ,'xp:',xp,'xs:',xs,'xn1:',xn1,'zn:',zn,'znn:',znn
!        a3=0.5
!  100   FORMAT(3(a,1x,e9.2)/a,5(1x,e9.2)/a,e9.2,/6(a,1x,e9.2)
!     &         /5(a,1x,e9.2)/)
!      ENDIF

      END
