      program veloc
c
c     FDE - Euler implicit scheme
c
      include 'params.inc'
      include 'pars.inc'
      include 'discrete.inc'

      real um,vis,y,tout,f,du1,du2
      real ap,as,an,a,s,q,u,uo,sp
      real kappa,viss,visn,k2,norm,epsilon
      real un,us,du,shear,yint,ypl
      real time,dt, mixf
      integer j,icwnd,icgrd,it,nout,l,inner,maxit,ndum,n

      parameter (kappa=0.41,k2=kappa*kappa)

      dimension ap(nj),as(nj),an(nj),y(nj),u(nj),uo(nj)
     &       ,a(nj),s(nj),q(nj), fy(nj)
      dimension shear(nj),ypl(nj)
      dimension tout(50),um(50)
      character filout*20,filin*20

      data s,uo,as,an,ap/nj*0.,nj*0.,nj*0.,nj*0.,nj*0./
c
c---- Title
      icwnd=33
      icgrd=34
c      write(*,*) ' NJM=',njm
      write(*,*) ' Enter input filename:'
      read(*,'(a)') filin
      open(icwnd,file='win.out',form='unformatted')

      open(icgrd,file='grid.bin',form='unformatted',status='unknown')
      read(icgrd) ndum
      read(icgrd) (y(j),j=1,ndum)
      read(icgrd) (fy(j),j=1,ndum)
      close(icgrd)

      open(55,file=filin,form='formatted',status='old')
      read(55,*) filout
      read(55,*) nout
      read(55,*) (tout(l),l=1,nout)
      read(55,*) (um(l),l=1,nout)
      read(55,*) fdens,visco, yzero
      read(55,*) maxit, epsilon, mixf
      read(55,*) press
      close(55)
      vis=visco/fdens

c      open(765,file='coef2.dat',form='formatted',status='unknown')

      it=0
      time=0.
c      sp=-press/fdens


  100 continue
      it=it+1
      dt=tout(it)-time
      time=time+dt

      do 21,j=2,njm
        s(j)=u(j)+max(0.,sp)*dt
   21 continue
      u(ndum)=um(it)

      inner=0
   99 continue
      inner=inner+1
      do 23,j=1,nj
   23     uo(j)=u(j)
c

c
c---- calculate coefficients
c     magnitude of updated u()
      norm=0.
c
c---  south boundary

      dun=2.*abs(u(3)-u(2))/(y(3)-y(1))
      dus=abs(u(2)-u(1))/(0.5*y(2)-y(1))
      
      visn=vis+k2*(y(2)+yzero)**2*dun
      viss=vis+k2*(y(1)+yzero)**2*dus
      an(2)=visn*dt/((y(2)-y(1))*(0.5*y(3)-y(1)))
      as(2)=viss*dt/((y(2)-y(1))*(0.5*y(2)-y(1)))
      ap(2)=1.+an(2)+as(2)

      do 11,j=3,njm
        viss=visn
        dun=2.*abs(u(j+1)-u(j))/(y(j+1)-y(j-1))
        visn=vis+k2*(y(j)+yzero)**2*dun
        an(j)=2.*visn*dt/((y(j+1)-y(j-1))*(y(j)-y(j-1)))
        as(j)=2.*viss*dt/((y(j)-y(j-2))*(y(j)-y(j-1)))
        ap(j)=1.+an(j)+as(j)
   11 continue

      call tdma(nj,u,ap,an,as,a,q,s)

      norm=0.
      do 77,j=1,nj
   77   norm=norm+(u(j)-uo(j))**2

      if (inner .eq. maxit .or. norm .lt. epsilon) goto 101
      goto 99 
  101 continue
      write(*,*) ' Norm:',norm,' inner:',inner
      if (it.lt.nout) goto 100      


      do 50,j=3,njm
        yint=.5*(y(j)+y(j-1))
        un=u(j+1)*fy(j)+u(j)*(1.-fy(j))
        us=u(j)*fy(j-1)+u(j-1)*(1.-fy(j-1))
        du=(un-us)/(y(j)-y(j-1))
        dun=abs(du)
        shear(j)=fdens*k2*yint**2*dun*du
   50 continue
c
      yint=0.5*y(2)+yzero
      du2=2.*(u(3)-u(2))/y(3)
      du1=2.*(u(2)-u(1))/(y(2)-y(1))
      du=mixf*du1+(1.-mixf)*du2
      dun=abs(du)
      shear(2)=fdens*k2*yint**2*dun*du
      shear(1)=visco*2.*u(2)/y(2)
      shear(nj)=shear(njm)


      call wrtphi(nj,u)
      call wrtphi(nj,ypl)
      call wrtphi(nj,shear)

      open(444,file=filout,form='unformatted')
      write(444) u,shear
      close(444)
      write(*,*) ' results written into ',filout
c      open(445,file='vel.dat',form='formatted')
c      do 69,i=2,nj
c        write(445,*) y(i),log(y(i)),u(i),shear(i)
c   69 continue
c      close(445)
c      write(*,*) ' u(NJM)=',u(njm)

      write(6,*) ' ustar= ',ustar

      end
c
c---  Matrix Loeser :  Thomas-Algorithmus
c
      subroutine tdma(n,u,ap,an,as,a,q,s)

      real ap,as,an,a,q,s,u,term
      integer n,njm,i
      dimension ap(n),an(n),as(n),s(n),u(n)
      dimension a(n),q(n)

      njm=n-1
      a(1)=0.
      q(1)=u(1)
      do 10 i=2,njm
         term=1./(ap(i)-as(i)*a(i-1))
         a(i)=an(i)*term
         q(i)=(s(i)+as(i)*q(i-1))*term
   10 continue
      do 15 i=njm,2,-1
   15 u(i)=a(i)*u(i+1)+q(i)
      return
      end
c
c---   Write-out routine
c
      subroutine wrtphi(n,f)
      integer n,ie,i,j
      real f
      dimension f(n)

      ie=n/6
      do 20,i=1,ie
        write(*,'(6(1x,e10.3))') (f(j),j=1+(i-1)*6,i*6)
   20 continue
      if (mod(n,6).ne.0) then
        write(*,'(6(1x,e10.3))') (f(j),j=1+ie*6,n)
      endif
      return
      end
      subroutine wrtphi2(n,f,title,chan)
      integer n,ie,i,j,chan
      character*6 title
      real f
      dimension f(n)

      ie=n/6
      write(chan,100) title
      do 20,i=1,ie
        write(chan,'(6(1x,e10.3))') (f(j),j=1+(i-1)*6,i*6)
   20 continue
      if (mod(n,6).ne.0) then
        write(chan,'(6(1x,e10.3))') (f(j),j=1+ie*6,n)
      endif
      write(chan,*)
  100 format(10x,10('*-'),a6,10('-*'))
      return
      end

