module  wind_boundary
  use precision
  use sysdep_io
  use maths_fnct
  implicit none
  private

  public :: read_wind_boundary_nl
  public :: write_wind_boundary_nl
  public :: init_wind_boundary
  public :: get_wind_boundary
  public :: boundary_is_stress
  type(fnct_t) :: decoded_fnct = null_fnct_t 

  type(fnct_pub_t) :: bound_fnct = fnct_pub_t("constant",0.4)
  logical          :: stress    = .true.  ! Is boundary condition stress or velocity

  namelist /wind_boundary_nl/  stress, bound_fnct

contains
  subroutine read_wind_boundary_nl(fp)
    integer, intent(in) :: fp
    read(fp, nml=wind_boundary_nl)
  end subroutine read_wind_boundary_nl
  subroutine write_wind_boundary_nl(fp)
    integer, intent(in) :: fp
    write(fp,'(A)')       ' &wind_boundary_nl'
    write(fp,'(A)',advance='no')'    bound_fnct     = '
    call write_fnct(fp,bound_fnct)
    write(fp,'(A,L1)')    '    stress     = ', stress
    write(fp,'(A)')       ' /'
    write(fp,'(A)')' '
 end subroutine write_wind_boundary_nl
  subroutine init_wind_boundary()
    decoded_fnct = init_fnct(bound_fnct)
  end subroutine init_wind_boundary
  ! Returns shear stress or velocity
  function get_wind_boundary(time) result(u)
    real(r_kind), intent(in) :: time
    real(r_kind) :: u 
    u = eval_fnct(decoded_fnct,time)
  end function get_wind_boundary
  function boundary_is_stress()
    logical boundary_is_stress
    boundary_is_stress = stress
  end function boundary_is_stress
end module wind_boundary
