
  interface profile_u
     module procedure profile_us, profile_ua
  end interface
  interface profile_du
     module procedure profile_dus, profile_dua
  end interface
  interface profile_f
     module procedure profile_fs, profile_fa
  end interface
  interface profile_df
     module procedure profile_dfs, profile_dfa
  end interface
  ! Returns analytic velocity profile
  ! x is height from surface
  ! xo is friction height
  ! fv is friction velocity
  ! k is von karmans constant
  ! myu is kinematic viscosity
  function profile_us(x,fv,x0,k,myu)
    real(r_kind), intent(in) :: x
    real(r_kind), intent(in) :: fv
    real(r_kind), intent(in) :: x0
    real(r_kind), intent(in) :: k
    real(r_kind), intent(in) :: myu
    real(r_kind) profile_us
    profile_us = fv*profile_f((2*k*fv/myu)*x)-fv*profile_f((2*k*fv/myu)*x0)
  end function profile_us
  function profile_ua(x,fv,x0,k,myu)
    real(r_kind), intent(in) :: x(:)
    real(r_kind), intent(in) :: fv
    real(r_kind), intent(in) :: x0
    real(r_kind), intent(in) :: k
    real(r_kind), intent(in) :: myu
    real(r_kind) profile_ua(size(x))
    profile_ua = fv*profile_f((2*k*fv/myu)*x)-fv*profile_f((2*k*fv/myu)*x0)
  end function profile_ua
  function profile_dus(x,fv,k,myu)
    real(r_kind), intent(in) :: x
    real(r_kind), intent(in) :: fv
    real(r_kind), intent(in) :: k
    real(r_kind), intent(in) :: myu
    real(r_kind) profile_dus
    profile_dus = 2*k*fv**2/myu*profile_df((2*k*fv/myu)*x)
  end function profile_dus
  function profile_dua(x,fv,k,myu)
    real(r_kind), intent(in) :: x(:)
    real(r_kind), intent(in) :: fv
    real(r_kind), intent(in) :: k
    real(r_kind), intent(in) :: myu
    real(r_kind) profile_dua(size(x))
    profile_dua = 2*k*fv**2/myu*profile_df((2*k*fv/myu)*x)
  end function profile_dua
  function profile_fa(x)
    real(r_kind), intent(in) :: x(:)
    real(r_kind) profile_fa(size(x))
    !where (x<100*epsilon(x))
    !   profile_fa = x/2-x**3/24+x**5/80-5*x**7/896
    !else
    where (x<1/sqrt(100*epsilon(x)))
       profile_fa = (1-sqrt(1+x**2))/x + atanh(x/sqrt(1+x**2))
    elsewhere ! Use asymptotic expansion
       profile_fa = -1+log(real(2,r_kind))+log(x)+1/x-.25/x**2 + 1/(32*x**4)-1/(96*x**6)
    end where
  end function profile_fa
  function profile_fs(x)
    real(r_kind), intent(in) :: x
    real(r_kind) profile_fs
    if (x<100*epsilon(x)) then
       profile_fs = x/2-x**3/24+x**5/80-5*x**7/896
    else if (x<1/sqrt(100*epsilon(x))) then
       profile_fs = (1-sqrt(1+x**2))/x + atanh(x/sqrt(1+x**2))
    else ! Use asymptotic expansion
       profile_fs = -1+log(real(2,r_kind))+log(x)+1/x-.25/x**2 + 1/(32*x**4)-1/(96*x**6)
    end if
  end function profile_fs
  function profile_dfa(x)
    real(r_kind), intent(in) :: x(:)
    real(r_kind) profile_dfa(size(x))
    profile_dfa = (sqrt(1+x**2)-1)/x**2
  end function profile_dfa
  function profile_dfs(x)
    real(r_kind), intent(in) :: x
    real(r_kind) profile_dfs
    profile_dfs = (sqrt(1+x**2)-1)/x**2
  end function profile_dfs
