! Keeps a 
module drag
  use sysdep_io
  use precision
  use string_param
  implicit none
  private

  public :: read_drag_nl
  public :: write_drag_nl
  public :: init_drag
  public :: drag_coeff

  integer :: decoded_type = 0 ! Decode the string for the function type
  integer, parameter :: NONE_T   = 1 ! No drag       Cd=0
  integer, parameter :: STOKES_T = 2 ! Stokes drag   Cd=24
  integer, parameter :: OSEEN_T  = 3 ! Oseen drag    Cd=24 +0.422Re
  integer, parameter :: TSUJI_T  = 4 ! 24 + 4*Re**0.687 
  integer, parameter :: CLIFT_T  = 5 ! 24 + 4*Re**0.687+Re*coeff_a/(1+42500*Re**(-1.16))
  integer, parameter :: MORSI_T  = 6 ! 24 + Re*(6/(1+sqrt(Re))+coeff_a)

  ! List of known function types
  type(param_table_t), parameter :: types(6) = (/&
       param_table_t("none",   NONE_T),&
       param_table_t("stokes", STOKES_T),&
       param_table_t("oseen",  OSEEN_T),&
       param_table_t("tsuji",  TSUJI_T),&
       param_table_t("clift",  CLIFT_T),&
       param_table_t("morsi",  MORSI_T)/)

  character(len=STRING_LENGTH) :: drag_type = 'morsi' 
  real(r_kind)                 :: coeff_a   = 0.422

  namelist /drag_nl/ drag_type, coeff_a

contains
  subroutine init_drag()
    decoded_type = decode_string_param(drag_type,types)
  end subroutine init_drag
  subroutine read_drag_nl(fp)
    integer, intent(in) :: fp
    integer iostat
read(fp,nml=drag_nl,err=1,iostat=iostat)
return
1 call print_iostat(iostat,std_err,'Error with drag_nl reading file "'//trim(get_name(fp))//'"')
  call error_halt
  end subroutine read_drag_nl
  subroutine write_drag_nl(fp)
    integer, intent(in) :: fp  
    write(fp,'(A)')       ' &drag_nl'
    write(fp,'(A)')      '    drag_type = "' // trim(drag_type) // '"'
    write(fp,'(A,G11.4)')'    coeff_a   = ', coeff_a
    write(fp,'(A)')       ' /'
    write(fp,'(A)')' '
  end subroutine write_drag_nl
  ! Returns the Re * Cd where Re is the Reynold number and Cd the drag coefficent
  function drag_coeff(Re) result(Cd)
    real(r_kind), intent(in) :: Re
    real(r_kind) Cd

   select case (decoded_type)
    case (NONE_T)
       Cd = 0
    case (STOKES_T)
       Cd = 24
    case (OSEEN_T)
       Cd = 24 + Re*coeff_a
    case (TSUJI_T)
       if (Re>1000) then
          Cd = Re*coeff_a
       else
          Cd = 24 + 4*Re**0.687
       end if
    case (CLIFT_T)
       cd = 24+4*Re**0.687 + Re*coeff_a/(1+42500*Re**(-1.16));
    case (MORSI_T)
       cd = 24 + Re*(6/(1+sqrt(Re))+coeff_a)
    case default
       call error_halt(&
            'drag.f90: Unknown drag function in drag_coeff()')
    end select
  end function drag_coeff
end module drag
