module grain_var
  use precision
  use maths_const
  use maths
  use random
  use sysdep_io
  implicit none
  private

  public :: random_grain_size
  public :: init_grain_var
  public :: end_grain_var
  public :: read_grain_nl
  public :: write_grain_nl

  integer, parameter :: MAX_GSZN = 100
  real(r_kind), allocatable, public :: grain_diam(:) ! Grain diameters
  real(r_kind), allocatable, public :: grain_dist(:) ! Cumulative distribution
  real(r_kind), allocatable, public :: grain_prob(:) ! Probability density

  ! Name list variables 
  real(r_kind), public  :: grain_dens           = 2650  
  real(r_kind), private :: grain_size(MAX_GSZN) = 2.5e-4
  integer, public       :: grain_size_num       = 1
  namelist /grain_nl/ grain_dens, grain_size, grain_size_num

  real(r_kind), parameter :: grav = 9.81
  ! sometimes we need scale variables representative off all the particles
  real(r_kind), public :: grain_len_scale
  real(r_kind), public :: grain_vel_scale
  real(r_kind), public :: grain_mass_scale
  real(r_kind), public :: grain_mom_scale
  real(r_kind), public :: grain_dens_scale
  real(r_kind), public :: grain_time_scale
  real(r_kind), public :: grain_min_mom
  real(r_kind), public :: grain_min_energy

contains 
  subroutine random_grain_size(diam)
    real(r_kind), intent(out) :: diam(:)
    diam = grain_diam(random_discrete(grain_dist,size(diam)))
  end subroutine random_grain_size
  subroutine read_grain_nl(fp)
    integer, intent(in) :: fp
    integer iostat
read(fp, nml=grain_nl,err=1,iostat=iostat)
return
1 call print_iostat(iostat,std_err,'Error with grain_nl reading file "'//trim(get_name(fp))//'"')
  call error_halt
  end subroutine read_grain_nl
  subroutine write_grain_nl(fp)
    integer, intent(in) :: fp
    write(fp,'(A)')' &grain_nl'
    write(fp,'(A,G11.4,A)')'    grain_dens     = ', grain_dens
    write(fp,'(A,I3,A)')    '    grain_size_num = ', grain_size_num
    write(fp,'(A,G11.4,A)')'    grain_size     = ', grain_size(1:grain_size_num)
    write(fp,'(A)')         ' /'
    write(fp,'(A)')' '
  end subroutine write_grain_nl
  subroutine init_grain_var()
    allocate(grain_diam(grain_size_num), grain_dist(grain_size_num), &
         grain_prob(grain_size_num))
    grain_diam = grain_size(1:grain_size_num)
    grain_prob = 1/grain_size_num
    grain_dist = cumsum(grain_prob)
    call assert(grain_dist(grain_size_num)==1,&
         'Grain size cumulative distribution does not sum to 1')

    grain_len_scale  = mean(grain_diam)
    grain_vel_scale  = sqrt(grav*grain_len_scale)
    grain_mass_scale = mean(grain_diam**3)*pi*grain_dens/6
    grain_dens_scale = grain_dens
    grain_time_scale = grain_len_scale/grain_vel_scale
    grain_mom_scale  = grain_vel_scale*grain_mass_scale
    grain_min_mom    = grain_mom_scale/2 ! Momentum that is too small to notice
    grain_min_energy = grain_min_mom**2/(2*grain_mass_scale)
  end subroutine init_grain_var
  subroutine end_grain_var()
    deallocate(grain_diam, grain_dist, grain_prob)
  end subroutine end_grain_var
end module grain_var
