module particle
  ! Module that defines the derived type for each grain

  use maths_const
  use vector
  !  include file defining the structure grains
  type particles           ! define a structure for particles
     logical      alive    ! does this grain exist, used for debugging
     integer      origin   ! How did the grain start
     integer      num      ! Each particle has a number
     integer      parent   ! Number of parent particle, 0 for aero
     real(r_kind) diam     ! particle diameter
     type(rvec_t) pos      ! position
     type(rvec_t) vel      ! velocity
     type(rvec_t) acc      ! acceleration
     real(r_kind) time     ! time particle is at
     real(r_kind) rc       ! reynold number = rc * vel
     real(r_kind) ff       ! Drag = mass*ff * urel * (24 + 4(rc*vel)^(.66))
     real(r_kind) mass     !
     integer      gy       ! contains grid height coordinate
     integer      gc       ! contains type of grain
     real(r_kind) max_y      ! Maximum height
     real(r_kind) start_t    ! Time at creation
     type(rvec_t) start_vel  ! velocity at creation
     type(rvec_t) start_pos  ! position at creation
  end type particles
  integer, parameter :: ORIGIN_DROP = 1     ! Origin was drop from top
  integer, parameter :: ORIGIN_INITIAL = 1  ! Grain was created initially
  integer, parameter :: ORIGIN_AERO = 1     ! Origin was aerodynamic entrainment
  integer, parameter :: ORIGIN_RICOCHET = 2 ! Origin was ricochet
  integer, parameter :: ORIGIN_SPLASH = 3   ! Origin was splash

  interface display_grain
     module procedure display_grain_s, display_grain_a
  end interface

contains
  ! Used for debugging
  ! Print everything out about a grain
  subroutine display_grain_s(fp,g)
    integer,        intent(in) :: fp
    type(particles), intent(in) :: g 
    if (.not.g%alive) write(fp,*)'Grain is dead'
    write(fp,*)'Grain number',g%num
    write(fp,'(a,2G11.4)')' position', g%pos
    write(fp,'(a,2G11.4)')' velocity', g%vel
    write(fp,'(a,2G11.4)')' acceleration', g%acc
    select  case (g%origin)
    case (ORIGIN_AERO)
       write(fp,*)'Aero'
    case (ORIGIN_RICOCHET)   
       write(fp,*)'Ricochet, parent',g%parent
    case (ORIGIN_SPLASH)
       write(fp,*)'Splash, parent',g%parent
    end select
    write(fp,'(2(a,G11.4))')' Start time',g%start_t,'current time',g%time 
    write(fp,'(a,G11.4)')   ' maxy y',g%max_y
    write(fp,'(a,2G11.4)')  ' Start position', g%start_pos
    write(fp,'(a,2G11.4)')  ' Start velocity', g%start_vel
  end subroutine display_grain_s
  subroutine display_grain_a(fp,g)
    integer,        intent(in) :: fp
    type(particles), intent(in) :: g(:)
    integer i
    do i=1,size(g)
       call display_grain_s(fp,g(i))
    end do
  end subroutine display_grain_a
end module particle
    
