module profile
  use precision
  use sysdep_io
  use wind
  use track_profile
  use impact_stats
  implicit none  
  private
  
  public :: read_profile_nl
  public :: write_profile_nl
  public :: init_profile
  public :: end_profile
  public :: write_profile
  public :: is_profile
  public :: start_profile

  logical :: is_profile          ! Are we profiling this time step

  integer :: profile_fp
 
  integer :: profile_its  = 10                ! How often to print a profile 
  logical :: profile_flag = .true.            ! Whether or to to print a profile
  logical :: append = .false.             ! Are we appending to a profile?
  logical :: snapshot = .true.  ! Profiles are whole time or just one time step?
  character(len=FILE_NAME_LEN) :: profile_fn = "profile.bin" 
  
  namelist /profile_nl/ profile_its, profile_flag, profile_fn, append, snapshot

contains
  subroutine read_profile_nl(fp)
    integer, intent(in) :: fp
    integer iostat
read(fp,nml=profile_nl,err=1,iostat=iostat)
return
1 call print_iostat(iostat,std_err,&
       'Error with profile_nl reading file "'//trim(get_name(fp))//'"')
  call error_halt
  end subroutine read_profile_nl
  subroutine write_profile_nl(fp)
    integer, intent(in) :: fp
    write(fp,'(A)')       ' &profile_nl'
    write(fp,'(A,I5)')    '    profile_its  = ', profile_its
    write(fp,'(A,L1)')    '    profile_flag = ', profile_flag
    write(fp,'(A,L1)')    '    append       = ', append
    write(fp,'(A,L1)')    '    snapshot     = ', snapshot
    write(fp,'(A)')       '    profile_fn   = "' // trim(profile_fn) // '"'
    write(fp,'(A)')       ' /'
    write(fp,'(A)')' '
  end subroutine write_profile_nl
  subroutine init_profile(outdir)
    character(len=*), intent(in) :: outdir
    if (.not. profile_flag) return
    call assign_lun(profile_fp) 
    if (append) then
       call err_open(profile_fp,file=trim(outdir)//trim(profile_fn),&
            form='unformatted',status='old',position='append')
    else
       call err_open(profile_fp,file=trim(outdir)//trim(profile_fn),&
            form='unformatted',status='replace')
    end if
    is_profile=.false.
  end subroutine  init_profile
  subroutine end_profile()
    if (.not. profile_flag) return
    call err_close(profile_fp);
    
    append = .true.
  end subroutine end_profile

  subroutine start_profile(iter, time)
    integer, intent(in) :: iter
    real(r_kind), intent(in) :: time    
    is_profile = (profile_flag .and. mod(iter,profile_its)==0)
    call start_track_profile(is_profile .or. snapshot)
    call start_impact_stats(is_profile .or. snapshot)
  end subroutine start_profile
  
  ! Called at the end of a time step
  subroutine write_profile(iter,time)
    integer, intent(in) :: iter
    real(r_kind), intent(in) :: time    
    if (is_profile .and. profile_flag) then
       write(profile_fp)iter,0,time
       call write_wind_profile(profile_fp)        ! Write wind profile stuff 
       call write_track_profile(profile_fp)  ! Write track profile stuff
    end if
  end subroutine write_profile         
end module profile
