# 1 "rmat_f90_def.F90"
# 1 "/home/jim/fortran/library/dimension.h" 1 
!Define preprocessor macros that define the dimension
!Default is Three dimensions
!Overide by compiling with -DTWODIM or defining 1 before this is included
!If the file is directly edited then the dependencies will cause the correct files
!to be recompiled so this is probably best
 
# 11


! Commands only for three dimensions use the macro CMD3() 
# 20


! Commands only for two dimensions use the macro CMD2()








# 2 "/home/jim/fortran/library/rmat_f90_def.F90" 2 
! Contains stuff pertaining to three dimensional matrices
module rmat_f90_def
  use precision
  use rvec_f90_def
  implicit none

  type, public :: rmat_t  ! Real(R_kind) matrix
     type(rvec_t) x ! Column 1
     type(rvec_t) y ! Column 2
      ! Column 3
  end type rmat_t

  type(rmat_t), parameter :: null_rmat=rmat_t(null_rvec,null_rvec)!SR2201BUG
!  type(rmat_t) :: null_rmat=rmat_t(rvec_t(0,0),rvec_t(0,0))

  
  interface assignment(=)
     module procedure rmat_equ_rmat, rmat_equ_r
  end interface
  interface operator(+)
     module procedure rmat_add_rmat, rmat_add_r, r_add_rmat 
  end interface
  interface operator(*)
     module procedure &
          rvec_mult_rvec, &
          rmat_mult_rvec, rmat_mult_arvec, &
          rvec_mult_rmat, arvec_mult_rmat, &
          rmat_mult_r, r_mult_rmat, rmat_mult_i, i_mult_rmat, &
          armat_mult_ar, ar_mult_armat, armat_mult_ai, ai_mult_armat
  end interface
  interface operator(/)
     module procedure &
          rmat_div_r, rmat_div_i, armat_div_r, armat_div_i, &
          armat_div_ar, armat_div_ai
  end interface
  interface sum
     module procedure armat_sum
  end interface
 interface transpose
     module procedure rmat_transpose
  end interface
contains
! Transpose a matrix
  function rmat_transpose(a) result(m)
    type(rmat_t), intent(in) :: a
    type(rmat_t) m
    m%x = rvec_t(a%x%x,a%y%x)
    m%y = rvec_t(a%x%y,a%y%x)
    
  end function rmat_transpose
! Matrix functions. Matrices are held as three column vectors
  function rvec_mult_rvec(a,b) result(r)
    type(rvec_t) , intent(in):: a,b
    type(rmat_t)  r
    r%x = b%x * a; r%y = b%y * a; 
  end function rvec_mult_rvec
  function rmat_mult_rvec(a,b) result(r)
    type(rmat_t), intent(in):: a
    type(rvec_t), intent(in):: b
    type(rvec_t)  r
    r = a%x * b%x + a%y * b%y 
  end function rmat_mult_rvec
  function rmat_mult_arvec(a,b) result(r)
    type(rmat_t), intent(in):: a
    type(rvec_t), intent(in):: b(:)
    type(rvec_t)  r(size(b))
    r = a%x * b%x + a%y * b%y 
  end function rmat_mult_arvec
  function rvec_mult_rmat(a,b) result(r)
    type(rvec_t), intent(in):: a
    type(rmat_t), intent(in):: b
    type(rvec_t)  r
    r = rvec_t(a .dot. b%x,a .dot. b%y)
  end function rvec_mult_rmat
  function arvec_mult_rmat(a,b) result(r)
    type(rvec_t), intent(in):: a(:)
    type(rmat_t), intent(in):: b
    type(rvec_t)  r(size(a))
    r%x = a .dot. b%x
    r%y = a .dot. b%y
    
  end function arvec_mult_rmat
  function rmat_add_rmat(a,b) result(m)
    type(rmat_t), intent(in):: a
    type(rmat_t), intent(in):: b
    type(rmat_t)  :: m 
    m = rmat_t(a%x + b%x,a%y + b%y)
  end function rmat_add_rmat
  function r_add_rmat(a,b) result(m)
    real(r_kind), intent(in) :: a
    type(rmat_t), intent(in):: b
    type(rmat_t)  m
    m = b
    m%x%x = m%x%x + a;m%y%y = m%y%y + a;
  end function r_add_rmat
  function rmat_add_r(a,b) result(m)
    type(rmat_t), intent(in):: a
    real(r_kind), intent(in) :: b
    type(rmat_t)  m
    m = a
    m%x%x = m%x%x + b;m%y%y = m%y%y + b;
  end function rmat_add_r
  subroutine rmat_equ_rmat(a,b)
    type(rmat_t), intent(out):: a
    type(rmat_t), intent(in):: b
    a%x=b%x;a%y=b%y;
  end subroutine rmat_equ_rmat
  subroutine rmat_equ_r(a,b)
    type(rmat_t), intent(out):: a
    real(r_kind), intent(in):: b
    a=rmat_t(rvec_t(b,0),rvec_t(0,b))
  end subroutine rmat_equ_r

  function r_mult_rmat(a,b) result(r)
    real(r_kind), intent(in):: a
    type(rmat_t), intent(in):: b
    type(rmat_t) r
    r%x=a*b%x;r%y=a*b%y;
  end function r_mult_rmat
  function rmat_mult_r(b,a) result(r)
    real(r_kind), intent(in):: a
    type(rmat_t), intent(in):: b
    type(rmat_t) r
    r%x=a*b%x;r%y=a*b%y;
  end function rmat_mult_r
   function rmat_mult_i(b,a) result(r)
    integer, intent(in):: a
    type(rmat_t), intent(in):: b
    type(rmat_t) r
    r%x=a*b%x;r%y=a*b%y;
  end function rmat_mult_i
   function i_mult_rmat(a,b) result(r)
    integer, intent(in):: a
    type(rmat_t), intent(in):: b
    type(rmat_t) r
    r%x=a*b%x;r%y=a*b%y;
  end function i_mult_rmat


  function ar_mult_armat(a,b) result(r)
    real(r_kind), intent(in):: a(:)
    type(rmat_t), intent(in):: b(size(a))
    type(rmat_t) r(size(a))
    r%x=a*b%x;r%y=a*b%y;
  end function ar_mult_armat
  function armat_mult_ar(a,b) result(r)
    type(rmat_t), intent(in):: a(:)
    real(r_kind), intent(in):: b(size(a))
    type(rmat_t) r(size(a))
    r%x=a%x*b;r%y=a%y*b;
  end function armat_mult_ar
   function armat_mult_ai(b,a) result(r)
    type(rmat_t), intent(in):: b(:)
    integer, intent(in):: a(size(b))
    type(rmat_t) r(size(b))
    r%x=a*b%x;r%y=a*b%y;
  end function armat_mult_ai
   function ai_mult_armat(a,b) result(r)
    integer, intent(in):: a(:)
    type(rmat_t), intent(in):: b(size(a))
    type(rmat_t) r(size(a))
    r%x=a*b%x;r%y=a*b%y;
  end function ai_mult_armat

  function rmat_div_r(a,b) result(r)
    type(rmat_t), intent(in):: a
    real(r_kind), intent(in):: b
    type(rmat_t) r
    r%x=a%x/b;r%y=a%y/b;
  end function rmat_div_r
  function rmat_div_i(a,b) result(r)
    type(rmat_t), intent(in):: a
    integer, intent(in):: b
    type(rmat_t) r
    r%x=a%x/b;r%y=a%y/b;
  end function rmat_div_i
   function armat_div_r(a,b) result(r)
    type(rmat_t), intent(in):: a(:)
    real(r_kind), intent(in):: b
    type(rmat_t) r(size(a))
    r%x=a%x/b;r%y=a%y/b;
  end function armat_div_r
  function armat_div_i(a,b) result(r)
    type(rmat_t), intent(in):: a(:)
    integer, intent(in):: b
    type(rmat_t) r(size(a))
    r%x=a%x/b;r%y=a%y/b;
  end function armat_div_i
   function armat_div_ar(a,b) result(r)
    type(rmat_t), intent(in):: a(:)
    real(r_kind), intent(in):: b(size(a))
    type(rmat_t) r(size(a))
    r%x=a%x/b;r%y=a%y/b;
  end function armat_div_ar
  function armat_div_ai(a,b) result(r)
    type(rmat_t), intent(in):: a(:)
    integer, intent(in):: b(size(a))
    type(rmat_t) r(size(a))
    r%x=a%x/b;r%y=a%y/b;
  end function armat_div_ai

  function armat_sum(a) result(r)
    type(rmat_t), intent(in):: a(:)
    type(rmat_t) r
    r%x=sum(a%x);r%y=sum(a%y);
  end function armat_sum
  function inertia_matrix(a,b) result(r)
    type(rvec_t), intent(in):: a(:)
    real(r_kind), intent(in), optional:: b(size(a))
    type(rmat_t) r
    if (present(b)) then
       r%x%x = sum(b*a%x**2)
       r%x%y = sum(b*a%x*a%y)
       
       r%y%x = r%x%y
       r%y%y = sum(b*a%y**2)
       
       
       
       
    else
       r%x%x = sum(a%x**2)
       r%x%y = sum(a%x*a%y)
       
       r%y%x = r%x%y
       r%y%y = sum(a%y**2)
       
       
       
       
    end if
  end function inertia_matrix
 end module rmat_f90_def
