# 1 "rreg_f90_def.F90"
# 1 "/home/jim/fortran/library/dimension.h" 1 
!Define preprocessor macros that define the dimension
!Default is Three dimensions
!Overide by compiling with -DTWODIM or defining 1 before this is included
!If the file is directly edited then the dependencies will cause the correct files
!to be recompiled so this is probably best
 
# 11


! Commands only for three dimensions use the macro CMD3() 
# 20


! Commands only for two dimensions use the macro CMD2()








# 2 "/home/jim/fortran/library/rreg_f90_def.F90" 2 
module rreg_f90_def
  use precision
  use rvec_f90_def
  implicit none
  type, public :: rreg_t   ! Real(R_kind) region
     type(rvec_t) min
     type(rvec_t) max
  end type rreg_t 

  type(rreg_t), parameter :: null_rreg = rreg_t(null_rvec,null_rvec) !SR2201BUG
!  type(rreg_t) :: null_rreg = rreg_t(rvec_t(0,0),rvec_t(0,0))

  interface assignment(=)
     module procedure rreg_equ_rreg
  end interface
  interface operator(<) 
     module procedure   le_rreg_rreg 
  end interface
  interface operator(>) 
     module procedure  ge_rreg_rreg
  end interface
  interface operator(<=) 
     module procedure  leq_rreg_rreg
  end interface
  interface operator(>=) 
     module procedure  geq_rreg_rreg
  end interface
  interface operator(*)
     module procedure r_mult_rreg 
  end interface
  interface volume
     module procedure rreg_volume
  end interface
  interface inside
     module procedure rreg_v_inside, rreg_av_inside
  end interface
  interface shrink
     module procedure rreg_shrink
  end interface
  interface grow
     module procedure rreg_grow
  end interface
  interface region
     module procedure rreg_region_rvecs,rreg_region_rveca
  end interface
! This is not a real union, but is the smallest cuboid region enclosing
! all the argument regions
  interface union 
     module procedure rreg_union_rreg2, rreg_union_rrega 
  end interface ! regions
  interface isfinite
     module procedure  isfinite_rreg, isfinite_arreg
  end interface

contains
! Functions for regions
! Convert a vector, vectors to a region
  function rreg_region_rvecs(b)  result(a)
    type(rvec_t),intent(in)  :: b
    type(rreg_t) :: a
    a%min = b
    a%max = b
  end function rreg_region_rvecs
  function rreg_region_rveca(b)  result(a)
    type(rvec_t),intent(in)  :: b(:)
    type(rreg_t) :: a
    a%min = minval(b);
    a%max = maxval(b)
  end function rreg_region_rveca
! union of regions, smallest region enclosing all regions
! Return the union of an array of regions
  function rreg_union_rrega(b)  result(a)
    type(rreg_t),intent(in)  :: b(:)
    type(rreg_t) :: a
    a%min = minval(b%min)
    a%max = maxval(b%max)
  end function rreg_union_rrega
  function rreg_union_rreg2(a,b) result (r)
    type(rreg_t),intent(in) :: a,b
    type(rreg_t) :: r
    r%min = min(a%min,b%min)
    r%max = max(a%max,b%max)
  end function rreg_union_rreg2  
  subroutine rreg_equ_rreg(a,b) ! Assignment of a region
    type(rreg_t),intent(out) :: a
    type(rreg_t),intent(in)  :: b
    a%min = b%min; a%max = b%max
  end subroutine rreg_equ_rreg
  function rreg_volume(a) ! Volume of a region
    type(rreg_t),intent(in) :: a
    real(r_kind) :: rreg_volume
    rreg_volume = volume(a%max-a%min)
  end function rreg_volume
! Which vectors from an array are inside a region
  function rreg_v_inside(a,b) ! Is a vector inside a region
    type(rreg_t),intent(in) :: a
    type(rvec_t), intent(in) :: b
    logical :: rreg_v_inside
    rreg_v_inside = a%min<=b .and. a%max>=b
  end function rreg_v_inside  ! Which vectors from an array are inside a region
  function rreg_av_inside(a,b)
    type(rreg_t),intent(in) :: a
    type(rvec_t), intent(in) :: b(:)
    logical :: rreg_av_inside(size(b))
    rreg_av_inside = a%min<=b .and. a%max>=b
  end function rreg_av_inside
  function rreg_grow(a,b)    ! Return a region increased by b
    type(rreg_t),intent(in) :: a
    real(r_kind), intent(in) :: b
    type(rreg_t) :: rreg_grow
    rreg_grow%min = a%min - scalar_to_rvec(b)
    rreg_grow%max = a%max + scalar_to_rvec(b)
  end function rreg_grow
  function rreg_shrink(a,b)    ! Return a region shrunk by b
    type(rreg_t),intent(in) :: a
    real(r_kind), intent(in) :: b
    type(rreg_t) :: rreg_shrink
    rreg_shrink%min = a%min + scalar_to_rvec(b)
    rreg_shrink%max = a%max - scalar_to_rvec(b)
  end function rreg_shrink
  function r_mult_rreg(a,b) result(rg)
    type(rreg_t),intent(in) :: b
    real(r_kind),intent(in) :: a
    type(rreg_t) :: rg
    rg%min = a * b%min; rg%max = a* b%max
  end function r_mult_rreg


  function le_rreg_rreg(a,b) result(r)!Is a region a contained in a region b
    type(rreg_t), intent(in) :: a
    type(rreg_t), intent(in) :: b
    logical :: r
    r = a%min>b%min .and. a%max<b%max
  end function le_rreg_rreg

  function ge_rreg_rreg(a,b) result(r)!Is a region a contained in a region b
    type(rreg_t), intent(in) :: a
    type(rreg_t), intent(in) :: b
    logical :: r
    r = a%min<b%min .and. a%max>b%max
  end function ge_rreg_rreg





   function leq_rreg_rreg(a,b) result(r)!Is a region a contained in a region b
    type(rreg_t), intent(in) :: a
    type(rreg_t), intent(in) :: b
    logical :: r
    r = a%min>=b%min .and. a%max<=b%max
  end function leq_rreg_rreg

  function geq_rreg_rreg(a,b) result(r)!Is a region a contained in a region b
    type(rreg_t), intent(in) :: a
    type(rreg_t), intent(in) :: b
    logical :: r
    r = a%min<=b%min .and. a%max>=b%max
  end function geq_rreg_rreg
 function isfinite_rreg(a) result(r)
    type(rreg_t), intent(in) :: a
    logical r
    r = isfinite(a%min) .and.  isfinite(a%max)
  end function isfinite_rreg
  function isfinite_arreg(a) result(r)
    type(rreg_t), intent(in) :: a(:)
    logical r
    r = isfinite(a%min) .and.  isfinite(a%max)
  end function isfinite_arreg

end module rreg_f90_def

