! Saltare
program saltare 
  use bed 
  use discrete
  use entrainment
  use grain_var 
  use maths
  use random
  use ricochet
  use saltare_var 
  use sample
  use splash
  use sysdep_io
  use timedate
  use track_grains
  use traj_samp
  use wind
  use profile
  use strings
  use drag
  use random_distrib
  use maths_fnct
  use ansi_strings
  use sysdep_io
  
  implicit none

  !call test_fnct()
  !call test_distrib()  ! Test probability distribution functions
  !  stop

  call main_prog()

contains
  subroutine main_prog()
    integer iostat
    character(len=FILE_NAME_LEN) fn             ! Name for input file
    character(len=5*FILE_NAME_LEN) input_str    ! Name for input file

    type(time_t) :: tstamp ! Starting time

    tstamp = get_time()    ! Keep track of how much time progam takes

    call disp_header()  ! display start up message

    ! Read the string of input files from std_input
    ! or use default
    ! Input file names should be separated by blanks
    write(std_out,'(1X,A)')&
         'Enter parameter file(s) separated by spaces. Blank for defaults.'
    write(std_out,'(A)',advance='no')'name(s): '
    read(std_in,'(A)',err=1,iostat=iostat) input_str
    do 
       call get_token(input_str, fn)        ! Get the next token
       if (len_trim(fn)==0) exit            ! Is it rmpty?
       call read_all(trim(fn))              ! Read in all the namelists
    end do

    call err_open(std_log,file=trim(outdir)//'salt_log')
    write(std_out,*)'Opened log file "', trim(outdir)//'salt_log"'

    call write_all(trim(outdir)//'param_pre.slt')
    call init_all()


    call main_iter(cum_iter, t_start, num_iter)


    call end_all() ! Save state and deallocate everything. Set flags for continueing

    tstamp = get_time()-tstamp            ! How much time elapsed this run
    call write_elapsed(std_out, tstamp, 'Elapsed time ')
    total_elap = total_elap + tstamp      ! How much time elapsed in total
    call write_elapsed(std_out, total_elap, 'Cumulative elapsed time ')

    ! Now produce a file to continue the simulation
    append = .true.
    call write_all(trim(outdir)//'param_cont.slt')
    write(std_out,*)"Program successfully completed."
    write(std_log,*)"Program successfully completed."
    return
1   call print_iostat(iostat,std_err,'Error reading standard input')
    call error_halt
  end subroutine main_prog

  ! Step forward for numiter from i_initial and t_initial

  subroutine main_iter(iter, time, num_iter)
    real(r_kind), intent(inout) :: time
    integer,      intent(inout) :: iter !How many iterations have been performed
    integer,      intent(in) :: num_iter !How many more to do

    integer i_initial
    real(r_kind) t_initial
    integer i

    t_initial = time
    i_initial = iter

    write(std_out,'(1X,2(A,I6))')&
         'Iterations from ',iter+1, ' to ', num_iter+iter
    write(std_out,'(1X,2(A,F8.3))')&
         'Times from ',t_initial, ' to ',t_initial + num_iter*t_step
    write(std_out,'(1X,A)')STR_YELLOW//'Starting calculation'//STR_RESET

    ! Write the initial state to p
    if (t_initial==0 .and. iter==0) then 
       call write_profile(iter,t_initial)     ! Write a profile
       call write_step(iter,t_initial)        ! Write step information
    end if

    do i = 1,num_iter
       iter = iter + 1
       time = t_initial + t_step*(iter-i_initial)

       call set_sample(iter) ! Set a flag saying whether we're sampling 
       call start_profile(iter,time)     ! start profile measurements
      
       ! Calculate the particle trajectories from time to time+t_step
       call lacalc_fnct(time,t_step)     ! Calculate particles
       call calc_wind(time,t_step)       ! calculate the wind velocity 
       call disp_step(std_out, iter, time) ! Display information about this step
       call write_profile(iter,time)     ! Write a profile
       call write_step(iter,time)        ! Write step information
    end do             ! end of time iterations
    write(std_out,*) STR_DOWN//STR_DOWN//STR_DOWN//STR_YELLOW//&
         'Iterations done. starting output process.'//STR_RESET
    write(std_out,*)'Final base stress',get_b_stress()
  end subroutine main_iter

 
  subroutine disp_header()  
    write(std_out,*)
    write(std_out,'(///,5(15x,a,/),/)')   &
         'Saltare 0.1, October 1998, Jim McElwaine'
  end subroutine disp_header 

  ! Call the functions to read everything in and display info if necessary
  subroutine read_all(fn)
    character(len=*), intent(in) :: fn
    integer fp

    write(std_out,*) 'Reading parameters from "',fn,'"'
    call assign_lun(fp)
    call err_open(fp, file=fn, status='old', action='read', delim='quote')
    call read_saltare_nl(fp)
    call read_profile_nl(fp)
    call read_grid_nl(fp)
    call read_sample_nl(fp)
    call read_track_nl(fp)
    call read_drag_nl(fp)
    call read_wind_nl(fp)
    call read_bed_nl(fp)
    call read_splash_nl(fp)
    call read_ricochet_nl(fp)
    call read_entrainment_nl(fp)
    call read_grain_nl(fp)
    call read_traj_samp_nl(fp)
    call err_close(fp)    
    
  end subroutine read_all

  subroutine write_all(fn)
    character(len=*), intent(in) :: fn
    integer fp
    write(std_out,*) 'Writing parameters to "',fn,'"'
    call assign_lun(fp)
    call err_open(fp, file=fn, status='replace', action='write', delim='quote')
    call write_saltare_nl(fp)
    call write_profile_nl(fp)
    call write_grid_nl(fp)
    call write_sample_nl(fp)
    call write_track_nl(fp)
    call write_drag_nl(fp)
    call write_wind_nl(fp)
    call write_bed_nl(fp)
    call write_splash_nl(fp)
    call write_ricochet_nl(fp)
    call write_entrainment_nl(fp)
    call write_grain_nl(fp)
    call write_traj_samp_nl(fp)
    call err_close(fp)
    
  end subroutine write_all

  subroutine init_all()
    call open_saltout(outdir)
    call init_grain_var()
    call init_profile(outdir) 
    call init_surface(outdir)    ! Read in the surface definitions
    call init_traj_samp(outdir)    ! Initialise track_grains

    call init_grid(outdir)      ! Now initialize extra grid info, air mass etc...
    call init_track_grains(outdir)    ! Initialise track_grains
    call init_sample()

    call init_splash()
    call init_ricochet()
    call init_entrainment()
    call init_drag()
    call init_wind(outdir)     ! Initialize the wind
  end subroutine init_all

  subroutine end_all()
    call finalize_traj_samp()
    call close_saltout()
    call end_splash()
    call end_ricochet()
    call end_profile()
    call end_wind(outdir)
    call end_entrainment()
    call end_grain_var()
    call end_track(outdir)
  end subroutine end_all

  subroutine write_step(iter,time)
    integer, intent(in) :: iter
    real(r_kind), intent(in) :: time
    write(saltout_fp,'(I6,F8.3)',advance='no')iter, time 
    call write_wind_step(saltout_fp)
    call write_track_step(saltout_fp)
    write(saltout_fp,*)
  end  subroutine write_step
  subroutine disp_step(fp,Xiter,Xtime)
    integer, intent(in) :: fp
    integer, intent(in) :: Xiter
    real(r_kind) , intent(in) :: Xtime

    if (screen) then
       write(fp,'(1X,A5,I6,A7,F8.3,51X)')&
            'iter ',Xiter,', time ',Xtime
       call display_wind_step(fp)
       call display_track_step(fp)
       write(fp,'(A)')STR_UP//STR_UP//STR_UP//STR_UP
    else
       write(fp,*)Xiter,STR_UP
   end if
  end subroutine disp_step
  subroutine open_saltout(dirn)
    character(len=*), intent(in) :: dirn
    if (append) then
       call err_open(saltout_fp,file=trim(dirn) // 'saltout.dat', &
            status='old',position='append')
    else
       call err_open(saltout_fp, file=trim(dirn) // 'saltout.dat', status='replace')
    end if
  end subroutine open_saltout
  subroutine close_saltout()
    call err_close(saltout_fp)
  end subroutine close_saltout
  
end program saltare
