! Name list for saltare.f90
module saltare_var
  use precision
  use sysdep_io
  use timedate

  implicit none
  integer saltout_fp ! File handle (unit number) for saltout file

  integer      :: num_iter = 500       ! How many iterations to perform
  real(r_kind) :: t_step    = 0.01      ! What is the timestep per iteration
  integer      :: histout  = 1         ! How often to print a history
  character(len=FILE_NAME_LEN) :: outdir= "out/" ! Directory name for output
  character(len=FILE_NAME_LEN) :: indir= "in/" ! Directory name for input
  
  logical :: screen     = .true.  ! Display information to the screen 
  logical :: history    = .true.  ! Flag whether or not to print a history
  logical :: append = .false. ! Append to output files don't overwrite

  integer      :: cum_iter = 0  ! How many iterations have already been performed
  real(r_kind) :: t_start  = 0  ! What time did the last iteration take the program to?
  type(time_t) :: total_elap = null_time_t ! How much time as this simualtion taken

  namelist /saltare_nl/ num_iter, cum_iter, t_step, t_start, append, histout, &
       screen, history, outdir, total_elap
 contains
  subroutine read_saltare_nl(fp)
    integer, intent(in) :: fp
    integer iostat
read(fp,nml=saltare_nl,err=1,iostat=iostat)
return
1 call print_iostat(iostat,std_err,&
       'Error with saltare_nl reading file "'//trim(get_name(fp))//'"')
  call error_halt
  end subroutine read_saltare_nl
  subroutine write_saltare_nl(fp)
    integer, intent(in) :: fp
    write(fp,'(A)')       ' &saltare_nl'
    write(fp,'(A,I6)')    '    num_iter   = ', num_iter
    write(fp,'(A,I6)')    '    cum_iter   = ', cum_iter
    write(fp,'(A,G11.4)') '    t_step     = ', t_step
    write(fp,'(A,G11.4)') '    t_start    = ', t_start
    write(fp,'(A,I5)')    '    histout    = ', histout
    write(fp,'(A,L1)')    '    screen     = ', screen
    write(fp,'(A,L1)')    '    history    = ', history
    write(fp,'(A,L1)')    '    append     = ', append
    write(fp,'(A)')       '    outdir     = "' // trim(outdir) // '"'
    write(fp,'(A)',advance='no')'    total_elap = '; write(fp,*)total_elap
    write(fp,'(A)')       ' /'
    write(fp,'(A)')' '
  end subroutine write_saltare_nl
end module saltare_var
