! Module which specifies when profiling and other files are to be written
! Default is to profile all time steps
module sample
  use precision
  use discrete
  use sysdep_io
  implicit none
  private

  public :: init_sample
  public :: is_sample
  public :: make_sample
  public :: clear_sample
  public :: read_sample_nl
  public :: write_sample_nl
  public :: set_sample

  integer :: samp_start = - huge(samp_start)! What iteration to start sampling at
  integer :: samp_end = huge(samp_start)! What iteration to stop sampling at
  integer samcnt ! How many samples have been performed

  integer, allocatable      :: anzahl(:)
  real(r_kind), allocatable :: konz(:)
  
  logical sample_true 
 
  namelist /sample_nl/ samp_start, samp_end 

contains
  subroutine init_sample()
    allocate(anzahl(ngr),konz(ngr))
    samcnt = 0
    anzahl = 0
    konz = 0
  end subroutine init_sample
  subroutine read_sample_nl(fp)
    integer, intent(in) :: fp
    integer iostat
    read(fp,nml=sample_nl,err=1,iostat=iostat)
    return
1   call print_iostat(iostat,std_err,&
         'Error with sample_nl reading file "'//trim(get_name(fp))//'"')
    call error_halt
  end subroutine read_sample_nl
  subroutine write_sample_nl(fp)
    integer, intent(in) :: fp
    write(fp,'(A)')       ' &sample_nl'
    write(fp,'(A,I11)')    '    samp_start = ', samp_start
    write(fp,'(A,I11)')    '    samp_end   = ', samp_end 
    write(fp,'(A)')       ' /'
    write(fp,'(A)')' '
  end subroutine write_sample_nl
  function is_sample() ! Should we make a sample on this iteration
    logical is_sample
    is_sample = sample_true
  end function is_sample
  subroutine set_sample(iter) ! Should we make a sample on this iteration
    integer, intent(in) :: iter
    sample_true = iter>=samp_start .and. iter<=samp_end
  end subroutine  set_sample
  subroutine make_sample(number,conc)
    integer, intent(in)  :: number(:)
    real(r_kind),intent(in) :: conc(:)
    anzahl=anzahl+number
    konz=konz+conc
    samcnt=samcnt+1
  end subroutine make_sample
  subroutine clear_sample()
    anzahl=0         ! for averag/writing (sampling over certain time)
    konz=0.          !                     dito
  end subroutine clear_sample
 end module sample
